package MongoInserter.Mong;

import java.time.LocalDateTime;
import java.util.HashMap;
import java.util.List;

import org.springframework.data.mongodb.core.MongoTemplate;
import org.springframework.data.mongodb.core.aggregation.Aggregation;
import org.springframework.data.mongodb.core.aggregation.AggregationResults;
import org.springframework.data.mongodb.core.aggregation.LookupOperation;
import org.springframework.data.mongodb.core.aggregation.UnwindOperation;
import org.springframework.data.mongodb.core.query.Criteria;

import com.mongodb.MongoClient;
import com.mongodb.MongoClientURI;

@SuppressWarnings("rawtypes")
public class MongoDBAggregates {
	private static MongoTemplate template = null;
	
	public void init() {
		try {
		    MongoClient mongoClient = new MongoClient(new MongoClientURI("mongodb://아이디:비번@127.0.0.1:27017/대상"));
			template = new MongoTemplate(mongoClient, "대상");
		} catch (Exception e) {
			e.printStackTrace();
		}
	}
	
	public static void main(String[] args) {		
		MongoDBAggregates app = new MongoDBAggregates();
		app.init();
		if(template != null){
			try {
				app.simpleUnwind();
				System.exit(1);
			} catch (Exception e) {
				e.printStackTrace();
			}
		}
	}

	public void simpleSingle(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;
		agg = Aggregation.newAggregation(
			Aggregation.project()  //1. 매핑할 이름
				.and("text").as("text_conv")  //2.기존 도큐먼트 이름 바꾸기 가능
				.and("type").as("type")
				.and("date").as("date")
				.and("number").as("number"),
			Aggregation.match(     //3. 날짜비교 
				new Criteria().andOperator(
						Criteria.where("date").gte(LocalDateTime.parse("2019-09-01T00:00:00")).lte(LocalDateTime.parse("2019-09-30T23:59:59"))
					)
			),
			Aggregation.group("type").count().as("total").sum("number").as("number")  //4.그룹핑할 대상, 각 대상의 총 합은 total로 한다. number는 합계
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
	public void simpleFirstAndLast(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;
		agg = Aggregation.newAggregation(
			Aggregation.project()  //1. 매핑할 이름
				.and("1").as("1")  //2. 1을 준 이유는 집계하는 대상이 1개면 불필요한 매핑이 되는 버그가 있다. 결과에는 아무런 문제 없다.
				.and("text").as("text_conv")  //3. 기존 도큐먼트 이름 바꾸기 가능
				.and("type").as("type")
				.and("date").as("date")
				.and("number").as("number"),
			Aggregation.match(     //4. 조건 null이 아닌것 
				new Criteria().andOperator(
						Criteria.where("text_conv").ne(null)
					)
			),
			Aggregation.group("type","1")                                         //5. type으로 그룹핑 하되, 
				.first("text_conv").as("text_conv").last("number").as("number")  //text는 처음 등록한 것만 가져오고, number는 나중에 등록한 것만 가져온다.
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
	
	public void simpleSubStringDateToString(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;
		agg = Aggregation.newAggregation(
			Aggregation.project()  //1. 매핑할 이름
				.and("1").as("1")  
				.and("text").substring(0,2).as("text_conv")  //2. 문자열 자르기 substring
				.and("type").as("type")
				.and("date").dateAsFormattedString("%Y-%d-%m").as("conv_date")  //날짜파싱
				.and("number").as("number"),
			Aggregation.match(     //3. 조건 null이 아닌것 
				new Criteria().andOperator(
						Criteria.where("text_conv").regex("")  //텍스트 전부 조건
					)
			),
			Aggregation.group("type","conv_date", "text_conv").count().as("count")
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
	public void simpleLookUp(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;
		
		LookupOperation lookUp = LookupOperation.newLookup()
				.from("head").localField("type")   //1. 묶을 컬렉션 이름은 head, 대상 도큐먼트는 같은 이름인 type
				.foreignField("type").as("ref");  //2. 조회할 컬렉션에서 해당 head 컬렉션이 묶일 도큐먼트 이름은 type, 별명은 ref
		agg = Aggregation.newAggregation(
			lookUp,  //3. Join 개념
			Aggregation.project()  //4. 매핑
				.and("type").as("type")
				.and("ref").as("ref")  
				.and("text").as("text"),
			Aggregation.match(    
				new Criteria().andOperator(
						Criteria.where("text").regex("")  //5. 텍스트 전부 조건
					)
			),
			Aggregation.group("type", "ref")
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
	public void simpleLookUp2(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;
		
		LookupOperation lookUp = LookupOperation.newLookup()
				.from("head").localField("type")   //1. 묶을 컬렉션 이름은 head, 대상 도큐먼트는 같은 이름인 type
				.foreignField("type").as("ref");  //2. 조회할 컬렉션에서 해당 head 컬렉션이 묶일 도큐먼트 이름은 type, 별명은 ref
		agg = Aggregation.newAggregation(
			lookUp,  //3. Join 개념
			Aggregation.project()  //4. 매핑
				.and("type").as("type")
				.and("ref").arrayElementAt(-1).as("ref1")  //5. 참조 컬랙션 가장 마지막 가져오기
				.and("ref").arrayElementAt(0).as("ref2")  //6. 참조 컬랙션 가장 처음 가져오기
				.and("text").as("text"),
			Aggregation.match(    
				new Criteria().andOperator(
						Criteria.where("text").regex("")  //7. 텍스트 전부 조건
					)
			),
			Aggregation.group("type", "ref1", "ref2")
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
	public void simpleUnwind(){
		Aggregation agg = null;
		AggregationResults<HashMap> results = null;

		UnwindOperation unwind = Aggregation.unwind("text");  //1.행으로 이루어진 대상을 1개의 열로 배열 형태로 풀 도큐먼트
		agg = Aggregation.newAggregation(
			unwind,
			Aggregation.project()  //2. 매핑
				.and("type").as("type")
				.and("1").as("1") 
				.and("text").as("text"),
			Aggregation.match(    
				new Criteria().andOperator(
						Criteria.where("text").regex("")  //3. 텍스트 전부 조건
					)
			),
			Aggregation.group("type", "1").push("text").as("text")  //4. push를 활용한 배열로 만든 text필드값 넣어주기
		);
		results = template.aggregate(agg, "body", HashMap.class);
		if(results != null){
			List<HashMap> target = results.getMappedResults();
			target.forEach(System.out::println);
		}
	}
	
}
